<!DOCTYPE html>
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<title>Skitai WSGI App Engine</title>
%raw
<style type="text/css"><!-- *{font-family:verdana,sans-serif;}body{margin:0;padding:0;background:#efefef;font-size:12px;color:#1e1e1e;} #titles{margin-left:0px;padding:10px;}#titles h1,h2{color: #000000;} #titles h1{color: #808080; font-size: 30px;} #content{min-height: 330px;padding:10px;background:#ffffff;} #error p,h3,b{font-size:11px;}#error{margin:0;padding:0;} hr{margin:0;padding:0;} #error hr{border-top:#888888 1px solid;} #error li,i{font-weight:normal;}#footer {font-size:9px;padding-left:10px;} pre{border:1px solid #888888;padding:7px;background:#efefef;margin:0 12px 12px 12px;} --></style>
%endraw
</head>
<body>
<div id="titles"><h1>Skitai WSGI App Engine</h1></div>
<hr />
<div id="content">
<div style="float: right; margin: 20px 10px 10px 20px;"><img src="/reindeer.jpg"></div>
<p>#content#</p>
</div> 
<hr />
<div id="footer">
<p>Generated Tue, 15 Dec 2015 06:14:39 GMT</p>
</div>
</body>
</html>
